﻿using SE.GOV.MM.Integration.Outlook.BusinessLayer.BusinessObject;
using SE.GOV.MM.Integration.Outlook.BusinessLayer.Helper;
using SE.GOV.MM.Integration.Outlook.Helper;
using SE.GOV.MM.Integration.Package.DataTransferObjects.BusinessObjects;
using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using System.Windows.Forms;

namespace SE.GOV.MM.Integration.Outlook
{
    /// <summary>
    /// Windows form that validates recipient in FaR and sends a message to Mina meddelanden.
    /// </summary>
    public partial class CreateMessageForm : Form
    {
        /// <summary>
        /// Senders mailadress
        /// </summary>
        private string _sender = string.Empty;

        /// <summary>
        /// Saves chosen signature to be able to replace if new signature is chosen. 
        /// </summary>
        private string _chosenSignature = string.Empty;
        
        /// <summary>
        /// Senders Outlook signature 
        /// </summary>
        private List<Signature> _signature = new List<Signature>();

        /// <summary>
        /// Keep track if sender checked if recipient is reachable.
        /// </summary>
        private bool ReachableRecipient { get; set; }

        public CreateMessageForm(string title, string sender) 
        {
            InitializeComponent();
            ActivateMessage(false);
            Load += CreateMessageForm_Load;
            Text = title;
            KeyDown += CreateMessageForm_KeyDown;
            KeyPreview = true;
            _sender = sender;
            GetSignatures();
        }

        /// <summary>
        /// Gets signatures saved in Outlook and displays in dropddownlist
        /// </summary>
        private void GetSignatures()
        {
            var signatureHelper = new SignatureHelper();
            this._signature = signatureHelper.ReadSignature();

            this._signature.Insert(0, new Signature());
            cmbxSignature.DataSource = this._signature;
            cmbxSignature.ValueMember = "Text";
            cmbxSignature.DisplayMember = "Name";
        }
        void CreateMessageForm_Load(object sender, EventArgs e)
        {
            this.tbxRecipient.Select();
        }

        /// <summary>
        /// Shows form as a dialog.
        /// </summary>
        public void ShowCreateMessageDialog()
        {
            this.ShowDialog();
        }

        /// <summary>
        /// Validates and sends the message to Mina meddelenden.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btn_CreateMessage_Click(object sender, EventArgs e)
        {       
            var mailHelper = new MailHelper();

            try
            {
                mailHelper.Validate(tbxRecipient.Text, tbx_Subject.Text);
                //Create MailItem
                var mailItem = mailHelper.CreateMailItem(_sender, tbxRecipient.Text, tbx_Subject.Text, tbxMessage.Text, ConfigHelper.ConfigurationEntity.Reference);
                //Add Attachments
                if (lbxAttachment.Items.Count > 0)
                {
                    mailItem.Attachments = new List<Attachment>();
                    var attachmentHelper = new AttachmentHelper();

                    foreach (ListItem item in lbxAttachment.Items)
                    {
                        mailItem.Attachments.Add(attachmentHelper.GetAttachment(item.Value));
                    }
                    attachmentHelper.ValidateAttachment(mailItem.Attachments);
                }

                //Check if sender really want to send message.
                if (MessageBox.Show(ResourceHelper.Resources[9].Text, ResourceHelper.Resources[1].Text, MessageBoxButtons.OKCancel, MessageBoxIcon.Information) == DialogResult.OK)
                {
                    var messageSender = new MessageSender(_sender);
                    var sentResult = messageSender.SendMailToPackage(mailItem);

                    if (sentResult.MessageSent)
                    {
                        MessageBox.Show(string.Format("Meddelandet är skickat via Mina meddelanden.{0}Observera att detta är ingen bekräftelse på att meddelandet har levererats till mottagarens brevlåda.", Environment.NewLine), ResourceHelper.Resources[1].Text, MessageBoxButtons.OK, MessageBoxIcon.Information);
                        this.Close();
                    }
                    else
                    {
                        MessageBox.Show(ResourceHelper.Resources[8].Text, ResourceHelper.Resources[1].Text, MessageBoxButtons.OK, MessageBoxIcon.Error);
                    }
                }

            }
            catch (ArgumentException ae) 
            {
                MessageBox.Show(ae.Message);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        /// <summary>
        /// Lookup in FaRService if recipient isReachable. 
        /// </summary>
        private void btnCheckRecipient_Click(object sender, EventArgs e)
        {
            try
            {
                if (!string.IsNullOrEmpty(tbxRecipient.Text))
                {
                    if (new RecipientHelper().CheckInvalidChars(tbxRecipient.Text))
                    {
                        if (ConfigHelper.ConfigurationEntity != null)
                        {
                            // Add century number to the recipient, 8101012386 -> 198101012386
                            var recipient = new RecipientHelper().AddCenturyNumbers(tbxRecipient.Text.Trim());
                            tbxRecipient.Text = recipient;
                            var validateIsOk = ValidateRecipient(recipient);

                            if (validateIsOk)
                            {
                                DisplayAndLookupIfRecipientIsReachable(recipient);
                            }
                            else
                            {
                                MessageBox.Show(ResourceHelper.Resources[3].Text, ResourceHelper.Resources[1].Text, MessageBoxButtons.OK, MessageBoxIcon.Error);
                            }
                        }
                        else
                        {
                            MessageBox.Show(ResourceHelper.Resources[10].Text, ResourceHelper.Resources[1].Text, MessageBoxButtons.OK, MessageBoxIcon.Error);
                        }
                    }
                    else
                    {
                        MessageBox.Show("Ogiltigt tecken för personnummer alt. organisationsnummer. Kontrollera och försök igen.", ResourceHelper.Resources[1].Text, MessageBoxButtons.OK, MessageBoxIcon.Error);
                    }
                }
                else
                {
                    MessageBox.Show(ResourceHelper.Resources[3].Text, ResourceHelper.Resources[1].Text, MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
            catch (Exception)
            {

            }
        }

        /// <summary>
        /// Activates/Deactivates the form until recipient is checked.
        /// </summary>
        private void ActivateMessage(bool activate)
        {
            ReachableRecipient = activate;
            tbxMessage.Enabled = activate;
            tbx_Subject.Enabled = activate;
            lbxAttachment.Enabled = activate;
            btnAddAttachment.Enabled = activate;
            btnRemoveAttachment.Enabled = activate;
            btn_CreateMessage.Enabled = false;
            tbxMessage.Text = string.Empty;
            tbx_Subject.Text = string.Empty;
            lbxAttachment.Items.Clear();
            cmbxSignature.Enabled = activate;
        }

        /// <summary>
        /// Shows a message in the form containing if a recipient is reachable and enables button. 
        /// </summary>
        /// <param name="recipient"></param>
        private void DisplayAndLookupIfRecipientIsReachable(string recipient)
        {
            //Lookup in FaR
            var result = IsReachable(recipient);

            if (result != null)
            {
                lbl_ResultText.Visible = true;
                var farhelper = new FaRServiceHelper();

                switch (result.IsReachable)
                {
                    case true:
                        lbl_ResultText.Text = farhelper.ReachableInFaRText(recipient);
                        ActivateMessage(true);
                        break;
                    case false:
                        lbl_ResultText.Text = farhelper.NotReachableInFaRText(recipient);
                        break;
                }
            }
            else
            {
                var farServiceHelper = new FaRServiceHelper();
                farServiceHelper.ExceptionWhenCheckingReachableInFaRDialog(recipient);
            }
        }

        /// <summary>
        /// If button to send message should activate.
        /// </summary>
        private void EnableSendMessage()
        {
            if (!string.IsNullOrEmpty(tbxRecipient.Text) && (!string.IsNullOrEmpty(tbx_Subject.Text)) && ReachableRecipient)
            {
                btn_CreateMessage.Enabled = true;
            }
            else
            {
                btn_CreateMessage.Enabled = false;
            }
        }

        /// <summary>
        /// Lookup in FaR if recipient isReachable for messages from Sender.
        /// </summary>
        /// <param name="recipient"></param>
        /// <returns></returns>
        private IsReachableResult IsReachable(string recipient)
        {
            var farhelper = new FaRServiceHelper();       
            return farhelper.IsReachableResult(recipient);
        }

        /// <summary>
        /// Validate recipient XXXXXXXX-XXXX, XXXXXXXXXXXX
        /// </summary>
        /// <param name="recipient"></param>
        /// <returns></returns>
        private bool ValidateRecipient(string recipient)
        {
            //Validate recipient
            var helper = new RecipientHelper();
            return helper.ValidRecipient(recipient);
        }


        /// <summary>
        /// if recipient is changed, check reset form.
        /// </summary>
        private void tbxRecipient_TextChanged(object sender, EventArgs e)
        {       
            lbl_ResultText.Text = string.Empty;
            DefaultAndClearForm();
        }

        /// <summary>
        /// If sender have checked isreachable, resets form
        /// </summary>
        private void DefaultAndClearForm()
        {
            if (ReachableRecipient)
            {
                ActivateMessage(false);
            }
        }

        /// <summary>
        /// Escape closes the form
        /// </summary>
        void CreateMessageForm_KeyDown(object sender, System.Windows.Forms.KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Escape)
            {
                Close();
            }
        }

        /// <summary>
        /// Closes the form.
        /// </summary>
        private void btnCancel_Click(object sender, EventArgs e)
        {
            Close();
        }

        /// <summary>
        /// Adds a attachment to the message
        /// </summary>
        private void btnAddAttachment_Click(object sender, EventArgs e)
        {
            var attachmentHelper = new AttachmentHelper();
            var dialog = new OpenFileDialog();
            dialog.Filter = attachmentHelper.GetAttachmentFilter(); //"Mina meddelanden |*.pdf;*.doc;*.docx;*.ics";// "pdf files (*.pdf)|*.pdf|doc files(*.doc)|*.doc|docx files(*.docx)|*.docx|ics files(*.ics)|*.ics";                                                                  
            dialog.Title = "Bifoga fil";
            dialog.Multiselect = true;
      
            if (dialog.ShowDialog() == DialogResult.OK)
            {
                if (dialog.FileNames.Length > 0)
                {
                    foreach (var filePath in dialog.FileNames)
                    {
                        //Add file to attachments
                        var fileInfo = new FileInfo(filePath);
                        var nameAndSize = GetAttachmentNameAndSize(fileInfo);
                        lbxAttachment.Items.Add(new ListItem(nameAndSize, fileInfo));
                    }
                }
            }
        }

        /// <summary>
        /// Concat name and size for a attachment.
        /// </summary>
        /// <param name="fileInfo"></param>
        /// <returns></returns>
        private string GetAttachmentNameAndSize(FileInfo fileInfo)
        {
            //count size as mb
            var size = Decimal.Parse(fileInfo.Length.ToString()) / 1024 / 1024; //KB //MB
            var roundSize = Decimal.Round(size, 2);
            return string.Format("{0} ({1}mb)",fileInfo.Name, roundSize.ToString());           
        }

        /// <summary>
        /// Remove a attachment from message
        /// </summary>
        private void btnRemoveAttachment_Click(object sender, EventArgs e)
        {
            if (lbxAttachment.SelectedItem != null)
            {
                lbxAttachment.Items.Remove(lbxAttachment.SelectedItem);
            }
        }

        /// <summary>
        /// Check if Send should be activated, when Subject text is changed
        /// </summary>
        private void tbx_Subject_TextChanged(object sender, EventArgs e)
        {
            EnableSendMessage();
        }

        /// <summary>
        /// When choosing a signature it adds directly to the textbox and removes the old one if it isnt changed.
        /// </summary>
        private void cmbxSignature_SelectedIndexChanged(object sender, EventArgs e)
        {
            //remove if signature is changed
            if (!string.IsNullOrEmpty(_chosenSignature))
            {
                tbxMessage.Text = tbxMessage.Text.Replace(_chosenSignature, string.Empty);
                //remove spaces
                tbxMessage.Text.Trim();
            }

            _chosenSignature = string.Empty;
            var sBuilder = new StringBuilder();
            sBuilder.Append(tbxMessage.Text);
            var signature = (Signature)cmbxSignature.SelectedItem;

            //Append new signature
            _chosenSignature = string.IsNullOrEmpty(signature.Text) ? string.Empty : signature.Text;
            sBuilder.Append(signature.Text);            
            tbxMessage.Text = sBuilder.ToString();
        }
    }
}
